@@ -427,28 +427,50 @@ add_item_acl (GnomeKeyringItem *item,
 static gboolean
 request_allowed_for_app (GnomeKeyringAccessRequest *request,
 			 GnomeKeyringApplicationRef *app_ref,
-			 GList *denied_keyrings)
+			 GList *denied_keyrings,
+			 gboolean *currently_asking)
 {
 	GnomeKeyringAccessControl *ac;
 	GList *l;
 
+	if (currently_asking) {
+		*currently_asking = FALSE;
+	}
+
 	if (request->granted) {
 		return TRUE;
 	}
 	
 	if (request->keyring != NULL) {
 		if (request->keyring->locked) {
+			if (currently_asking) {
+				*currently_asking = request->keyring->asking_password;
+			}
 			return FALSE;
 		}
 		/* TODO: verify app ACL vs keyring?? */
 		return TRUE;
+
 	} else if (request->item != NULL) {
 		if (request->item->locked) {
+			if (currently_asking) {
+				*currently_asking = request->item->keyring->asking_password;
+			}
 			return FALSE;
 		}
+
+		/* Is it only basic info read access (no secret)? */
+		if(request->request_type == GNOME_KEYRING_ACCESS_REQUEST_ITEM && 
+		   request->access_type == GNOME_KEYRING_ACCESS_READ) {	
+
+			/* If item is not marked as an application secret then automatic access */
+			if ((request->item->type & GNOME_KEYRING_ITEM_APPLICATION_SECRET) == 0)
+				return TRUE;
+		}
+			
+		/* Otherwise full access to secret, or modify access we check the calling application */
 		for (l = request->item->acl; l != NULL; l = l->next) {
 			ac = l->data;
-			
 			if (app_ref_match (app_ref, ac->application) &&
 			    (ac->types_allowed & request->access_type) == request->access_type) {
 				return TRUE;
@@ -462,7 +484,21 @@ request_allowed_for_app (GnomeKeyringAccessRequest *request,
 static void
 gnome_keyring_ask_kill (GnomeKeyringAsk *ask)
 {
+	GnomeKeyring *keyring;
+	
 	if (ask->input_watch != 0) {
+
+		if (ask->current_ask_type == ASK_KEYRING_PASSWORD) {
+			if (ask->current_request->keyring != NULL) {
+				keyring = ask->current_request->keyring;
+			} else {
+				keyring = ask->current_request->item->keyring;
+			}
+			if (keyring) {
+				keyring->asking_password = FALSE;
+			}
+		}
+		
 		g_source_remove (ask->input_watch);
 		ask->input_watch = 0;
 	}
@@ -539,12 +575,23 @@ match_attributes (GnomeKeyringItem *item,
 }
 
 static GnomeKeyringAccessRequest *
-access_request_from_item (GnomeKeyringItem *item,
-			  GnomeKeyringAccessType access_type)
+access_request_from_item (GnomeKeyringItem *item)
 {
 	GnomeKeyringAccessRequest *access_request;
 	access_request = g_new0 (GnomeKeyringAccessRequest, 1);
 	access_request->request_type = GNOME_KEYRING_ACCESS_REQUEST_ITEM;
+	access_request->access_type = GNOME_KEYRING_ACCESS_READ; /* Always only read access */
+	access_request->item = item;
+	return access_request;
+}
+
+static GnomeKeyringAccessRequest *
+access_request_from_item_with_secret (GnomeKeyringItem *item,
+			  	      GnomeKeyringAccessType access_type)
+{
+	GnomeKeyringAccessRequest *access_request;
+	access_request = g_new0 (GnomeKeyringAccessRequest, 1);
+	access_request->request_type = GNOME_KEYRING_ACCESS_REQUEST_ITEM_SECRET;
 	access_request->access_type = access_type;
 	access_request->item = item;
 	return access_request;
@@ -1257,8 +1304,8 @@ op_list_items_execute (GString *packet,
 	char *keyring_name;
 	GnomeKeyringOpCode opcode;
 	GnomeKeyringItem *item;
-	GList *l;
-	GnomeKeyringAccessRequest *req;
+	GList *l, *items;
+	GnomeKeyringAccessRequest *req, *list_req;
 	
 	if (!gnome_keyring_proto_decode_op_string (packet,
 						   &opcode,
@@ -1285,11 +1332,32 @@ op_list_items_execute (GString *packet,
 			gnome_keyring_proto_add_uint32 (result, 0);
 		} else {
 			gnome_keyring_proto_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
-			gnome_keyring_proto_add_uint32 (result, g_list_length (keyring->items));
+
+			/* 
+			 * Request access to these items. This cannot be done earlier in 
+			 * op_list_items_collect because the keyring may have been locked.
+			 */
+			list_req = access_request_from_item (NULL);
+			
+			items = NULL;
 			for (l = keyring->items; l != NULL; l = l->next) {
+				list_req->item = l->data;
+				list_req->granted = FALSE;
+				if (request_allowed_for_app (list_req, app_ref, NULL, NULL))
+					items = g_list_prepend (items, list_req->item);
+			}
+			items = g_list_reverse (items);
+
+			gnome_keyring_access_request_free (list_req);
+
+			/* Send the results */			
+			gnome_keyring_proto_add_uint32 (result, g_list_length (items));
+			for (l = items; l != NULL; l = l->next) {
 				item = l->data;
 				gnome_keyring_proto_add_uint32 (result, item->id);
 			}
+
+			g_list_free (items);
 		}
 	}
 	
@@ -1341,11 +1409,11 @@ op_create_item_collect (GString *packet,
 
 		for (ilist = keyring->items; ilist != NULL; ilist = ilist->next) {
 			item = ilist->data;
-			if (item->type == type &&
+			if ((item->type & GNOME_KEYRING_ITEM_TYPE_MASK) == (type & GNOME_KEYRING_ITEM_TYPE_MASK) &&
 			    match_attributes (item, keyring->locked ? hashed : attributes, TRUE)) {
 				found_existing = TRUE;
 				access_request =
-					access_request_from_item (item, GNOME_KEYRING_ACCESS_WRITE);
+					access_request_from_item_with_secret (item, GNOME_KEYRING_ACCESS_WRITE);
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
 				break;
@@ -1420,7 +1488,7 @@ op_create_item_execute (GString *packet,
 		goto bail;
 	}
 
-	if (type >= GNOME_KEYRING_ITEM_LAST_TYPE ||
+	if ((type & GNOME_KEYRING_ITEM_TYPE_MASK) >= GNOME_KEYRING_ITEM_LAST_TYPE ||
 	    display_name == NULL ||
 	    secret == NULL) {
 		res = GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
@@ -1445,6 +1513,9 @@ op_create_item_execute (GString *packet,
 		goto bail;
 	}
 
+	/* Copy in item type flags */
+	item->type |= (type & ~GNOME_KEYRING_ITEM_TYPE_MASK);
+
 	g_free (item->display_name);
 	item->display_name = g_strdup (display_name);
 	g_free (item->secret);
@@ -1501,7 +1572,7 @@ op_delete_item_collect (GString *packet,
 			item = find_item (keyring, item_id);
 			if (item != NULL) {
 				access_request =
-					access_request_from_item (item,
+					access_request_from_item_with_secret (item,
 								  GNOME_KEYRING_ACCESS_REMOVE);
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
@@ -1576,22 +1647,18 @@ op_delete_item_execute (GString *packet,
 
 
 static gboolean
-op_get_item_info_or_attributes_collect (GString *packet,
-					GList **access_requests_out)
+op_get_item_info_collect (GString *packet, GList **access_requests_out)
 {
 	char *keyring_name;
 	GnomeKeyring *keyring;
 	GnomeKeyringItem *item;
 	GnomeKeyringOpCode opcode;
-	guint32 item_id;
+	guint32 item_id, flags;
 	GnomeKeyringAccessRequest *access_request;
 	GList *access_requests;
 
-	
-	if (!gnome_keyring_proto_decode_op_string_int (packet,
-						       &opcode,
-						       &keyring_name,
-						       &item_id)) {
+	if (!gnome_keyring_proto_decode_get_item_info (packet, &opcode, &keyring_name, 
+						       &item_id, &flags)) {
 		return FALSE;
 	}
 
@@ -1601,9 +1668,13 @@ op_get_item_info_or_attributes_collect (GString *packet,
 		if (keyring != NULL) {
 			item = find_item (keyring, item_id);
 			if (item != NULL) {
-				access_request =
-					access_request_from_item (item,
-								  GNOME_KEYRING_ACCESS_READ);
+				/* Request access based on what parts were desired */
+				if ((flags & GNOME_KEYRING_ITEM_INFO_SECRET) == GNOME_KEYRING_ITEM_INFO_SECRET) {
+					access_request = access_request_from_item_with_secret (item,
+									  GNOME_KEYRING_ACCESS_READ);
+				} else {
+					access_request = access_request_from_item (item);
+				}
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
 			}
@@ -1623,17 +1694,15 @@ op_get_item_info_execute (GString *packet,
 			  GnomeKeyringApplicationRef *app_ref,
 			  GList *access_requests)
 {
-	char *keyring_name;
+	char *keyring_name, *secret;
 	GnomeKeyring *keyring;
 	GnomeKeyringItem *item;
 	GnomeKeyringOpCode opcode;
-	guint32 item_id;
+	guint32 item_id, flags;
 	GnomeKeyringAccessRequest *access_request;
 	
-	if (!gnome_keyring_proto_decode_op_string_int (packet,
-						       &opcode,
-						       &keyring_name,
-						       &item_id)) {
+	if (!gnome_keyring_proto_decode_get_item_info (packet, &opcode, &keyring_name,
+						       &item_id, &flags)) {
 		return FALSE;
 	}
 
@@ -1668,9 +1737,19 @@ op_get_item_info_execute (GString *packet,
 	if (!gnome_keyring_proto_add_utf8_string (result, item->display_name)) {
 		return FALSE;
 	}
-	if (!gnome_keyring_proto_add_utf8_string (result, item->secret)) {
+
+	/* Only return the secret if it was requested */
+	secret = NULL;
+	if ((flags & GNOME_KEYRING_ITEM_INFO_SECRET) == GNOME_KEYRING_ITEM_INFO_SECRET) {
+		g_assert(access_request->request_type == GNOME_KEYRING_ACCESS_REQUEST_ITEM_SECRET);
+		secret = item->secret;
+	}
+
+	/* Always put the secret string or NULL in the results for compatibility */
+	if (!gnome_keyring_proto_add_utf8_string (result, secret)) {
 		return FALSE;
 	}
+
 	gnome_keyring_proto_add_time (result, keyring->mtime);
 	gnome_keyring_proto_add_time (result, keyring->ctime);
 	
@@ -1680,6 +1759,46 @@ out:
 	return TRUE;
 }
 
+static gboolean
+op_get_item_acl_or_attributes_collect (GString *packet,
+					GList **access_requests_out)
+{
+	char *keyring_name;
+	GnomeKeyring *keyring;
+	GnomeKeyringItem *item;
+	GnomeKeyringOpCode opcode;
+	guint32 item_id;
+	GnomeKeyringAccessRequest *access_request;
+	GList *access_requests;
+
+	
+	if (!gnome_keyring_proto_decode_op_string_int (packet,
+						       &opcode,
+						       &keyring_name,
+						       &item_id)) {
+		return FALSE;
+	}
+
+	access_requests = NULL;
+	if (keyring_name != NULL) {
+		keyring = find_keyring (keyring_name);
+		if (keyring != NULL) {
+			item = find_item (keyring, item_id);
+			if (item != NULL) {
+				access_request = access_request_from_item (item);
+				access_requests = g_list_prepend (access_requests,
+								  access_request);
+			}
+		}
+	}
+
+	*access_requests_out = access_requests;
+	g_free (keyring_name);
+	
+	return TRUE;
+	
+}
+
 static gboolean
 op_get_item_attributes_execute (GString *packet,
 				GString *result,
@@ -1852,9 +1971,6 @@ out:
 	return TRUE;
 }
 
-
-
-
 static gboolean
 op_set_item_info_or_attributes_collect (GString *packet,
 					GList **access_requests_out)
@@ -1882,7 +1998,7 @@ op_set_item_info_or_attributes_collect (GString *packet,
 			item = find_item (keyring, item_id);
 			if (item != NULL) {
 				access_request =
-					access_request_from_item (item,
+					access_request_from_item_with_secret (item,
 								  GNOME_KEYRING_ACCESS_WRITE);
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
@@ -1947,7 +2063,7 @@ op_set_item_info_execute (GString *packet,
 
 	gnome_keyring_proto_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
 
-	if (type != GNOME_KEYRING_ITEM_NO_TYPE) {
+	if ((type & GNOME_KEYRING_ITEM_TYPE_MASK) != GNOME_KEYRING_ITEM_NO_TYPE) {
 		item->type = type;
 	}
 	if (item_name != NULL) {
@@ -2154,7 +2270,7 @@ op_find_execute (GString *packet,
 		access_request = l->data;
 		item = access_request->item;
 		if (item != NULL &&
-		    item->type == type &&
+		    (item->type & GNOME_KEYRING_ITEM_TYPE_MASK) == (type & GNOME_KEYRING_ITEM_TYPE_MASK) &&
 		    !item->locked &&
 		    match_attributes (item, attributes, FALSE)) {
 			if (!gnome_keyring_proto_add_utf8_string (result, item->keyring->keyring_name)) {
@@ -2210,10 +2326,10 @@ op_find_collect (GString *packet,
 		keyring = klist->data;
 		for (ilist = keyring->items; ilist != NULL; ilist = ilist->next) {
 			item = ilist->data;
-			if (item->type == type &&
+			if ((item->type & GNOME_KEYRING_ITEM_TYPE_MASK) == (type & GNOME_KEYRING_ITEM_TYPE_MASK) &&
 			    match_attributes (item, keyring->locked ? hashed : attributes, FALSE)) {
 				access_request =
-				  access_request_from_item (item, GNOME_KEYRING_ACCESS_READ);
+				  access_request_from_item_with_secret (item, GNOME_KEYRING_ACCESS_READ);
 				access_requests = g_list_prepend (access_requests,
 								  access_request);
 			}
@@ -2237,6 +2353,17 @@ finish_ask_io (GnomeKeyringAsk *ask,
 	GnomeKeyring *keyring;
 	GnomeKeyringItem *item;
 
+	if (ask->current_ask_type == ASK_KEYRING_PASSWORD) {
+		if (ask->current_request->keyring != NULL) {
+			keyring = ask->current_request->keyring;
+		} else {
+			keyring = ask->current_request->item->keyring;
+		}
+		if (keyring) {
+			keyring->asking_password = FALSE;
+		}
+	}
+	    
 	ask->input_watch = 0;
 	ask->ask_pid = 0;
 	
@@ -2369,6 +2496,7 @@ launch_ask_helper (GnomeKeyringAsk *ask,
 {
 	GnomeKeyringAccessRequest *request;
 	GnomeKeyringApplicationRef *app_ref;
+	GnomeKeyring *keyring;
 	GIOChannel *channel;
 	char **envp;
 	int i, n;
@@ -2400,11 +2528,15 @@ launch_ask_helper (GnomeKeyringAsk *ask,
 	if (app_ref->pathname != NULL) {
 		envp[i++] = g_strdup_printf("KEYRING_APP_PATH=%s", ask->app_ref->pathname);
 	}
+	keyring = NULL;
 	if (request->item != NULL) {
 		envp[i++] = g_strdup_printf("KEYRING_NAME=%s", request->item->keyring->keyring_name);
 		envp[i++] = g_strdup_printf("ITEM_NAME=%s", request->item->display_name);
+		keyring = request->item->keyring;
 	} else if (request->keyring != NULL) {
 		envp[i++] = g_strdup_printf("KEYRING_NAME=%s", request->keyring->keyring_name);
+		request->keyring->asking_password = TRUE;
+		keyring = request->keyring;
 	} else  if (request->new_keyring != NULL) {
 		envp[i++] = g_strdup_printf("KEYRING_NAME=%s", request->new_keyring);
 	}
@@ -2442,6 +2574,9 @@ launch_ask_helper (GnomeKeyringAsk *ask,
 				      &stdout_fd,
 				      NULL,
 				      &error)) {
+		if (keyring && ask_type == ASK_KEYRING_PASSWORD) {
+			keyring->asking_password = TRUE;
+		}
 		channel = g_io_channel_unix_new (stdout_fd);
 		ask->input_watch = g_io_add_watch (channel, G_IO_IN | G_IO_HUP,
 						   ask_io, ask);
@@ -2457,77 +2592,89 @@ launch_ask_helper (GnomeKeyringAsk *ask,
 static void
 schedule_ask (GnomeKeyringAsk *ask)
 {
-	GnomeKeyringAccessRequest *request;
+	GnomeKeyringAccessRequest *request = ask->current_request;
+	gboolean app_secret = FALSE;
+	gboolean deny = FALSE;
+	gboolean iterate = FALSE;
+
+	if(request->item) {
+		app_secret = (request->item->type & GNOME_KEYRING_ITEM_APPLICATION_SECRET) == 
+					GNOME_KEYRING_ITEM_APPLICATION_SECRET;
+	}
 
-	request = ask->current_request;
 	switch (request->request_type) {
 	case GNOME_KEYRING_ACCESS_REQUEST_KEYRING:
-		if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD))
+			deny = iterate = TRUE;
 		break;
+
 	case GNOME_KEYRING_ACCESS_REQUEST_ITEM:
 		if (request->item->keyring->locked) {
-			if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD)) {
-				/* no way to allow request, denying */
-				ask->access_requests = g_list_remove (ask->access_requests, request);
-				gnome_keyring_access_request_free (request);
-				/* iterate */
-				gnome_keyring_ask_iterate (ask);
-			}
+			if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD)) 
+				deny = iterate = TRUE;
+
 		} else {
-			if (!launch_ask_helper (ask, ASK_ITEM_READ_WRITE_ACCESS)) {
-				/* no way to allow request, denying */
-				ask->access_requests = g_list_remove (ask->access_requests, request);
-				gnome_keyring_access_request_free (request);
-				/* iterate */
-				gnome_keyring_ask_iterate (ask);
-			}
+			/* We never prompt for simple read requests, they're either allowed or not */
+			request->granted = !app_secret;
+			deny = app_secret;
+			iterate = TRUE;
 		}
 		break;
+
+	case GNOME_KEYRING_ACCESS_REQUEST_ITEM_SECRET:
+		if (request->item->keyring->locked) {
+			if (!launch_ask_helper (ask, ASK_KEYRING_PASSWORD)) 
+				deny = iterate = TRUE;
+
+		} else if (app_secret) {
+			/* We never allow access to 'application' secrets from the wrong app */
+			deny = iterate = TRUE;
+
+		} else {
+			if (!launch_ask_helper (ask, ASK_ITEM_READ_WRITE_ACCESS)) 
+				deny = iterate = TRUE;
+		}
+		break;
+
 	case GNOME_KEYRING_ACCESS_REQUEST_NEW_KEYRING_PASSWORD:
-		if (!launch_ask_helper (ask, ASK_NEW_KEYRING_PASSWORD)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_NEW_KEYRING_PASSWORD)) 
+			deny = iterate = TRUE;
 		break;	
+
 	case GNOME_KEYRING_ACCESS_REQUEST_ORIGINAL_CHANGE_KEYRING_PASSWORD:
-		if (!launch_ask_helper (ask, ASK_ORIGINAL_CHANGE_KEYRING_PASSWORD)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_ORIGINAL_CHANGE_KEYRING_PASSWORD))
+			deny = iterate = TRUE;
 		break;
+
 	case GNOME_KEYRING_ACCESS_REQUEST_CHANGE_KEYRING_PASSWORD:
-		if (!launch_ask_helper (ask, ASK_CHANGE_KEYRING_PASSWORD)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_CHANGE_KEYRING_PASSWORD))
+			deny = iterate = TRUE;
 		break;
+
 	case GNOME_KEYRING_ACCESS_REQUEST_DEFAULT_KEYRING:
-		if (!launch_ask_helper (ask, ASK_DEFAULT_KEYRING)) {
-			/* no way to allow request, denying */
-			ask->access_requests = g_list_remove (ask->access_requests, request);
-			gnome_keyring_access_request_free (request);
-			/* iterate */
-			gnome_keyring_ask_iterate (ask);
-		}
+		if (!launch_ask_helper (ask, ASK_DEFAULT_KEYRING))
+			deny = iterate = TRUE;
 		break;
+
 	default:
 		g_assert_not_reached ();
 	}
+
+	if (deny) {
+		ask->access_requests = g_list_remove (ask->access_requests, request);
+		gnome_keyring_access_request_free (request);
+	}
+
+	if (iterate) 
+		gnome_keyring_ask_iterate (ask);
+
+}
+
+static gboolean
+idle_ask_iterate (gpointer data)
+{
+	gnome_keyring_ask_iterate ((GnomeKeyringAsk *)data);
+	return FALSE;
 }
 
 static gboolean
@@ -2535,6 +2682,7 @@ gnome_keyring_ask_iterate (GnomeKeyringAsk *ask)
 {
 	GnomeKeyringAccessRequest *unfulfilled_request;
 	GnomeKeyringAccessRequest *request;
+	gboolean currently_asking;
 	GList *l;
 
  restart:
@@ -2545,7 +2693,12 @@ gnome_keyring_ask_iterate (GnomeKeyringAsk *ask)
 	for (l = ask->access_requests; l != NULL; l = l->next) {
 		request = l->data;
 
-		if (!request_allowed_for_app (request, ask->app_ref, ask->denied_keyrings)) {
+		if (!request_allowed_for_app (request, ask->app_ref, ask->denied_keyrings, &currently_asking)) {
+			if (currently_asking) {
+				/* Sleep until dialog is hopefully finished and try again */
+				g_timeout_add (400, idle_ask_iterate, ask);
+				return FALSE;
+			}
 			unfulfilled_request = request;
 			break;
 		}
@@ -2571,14 +2724,6 @@ gnome_keyring_ask_iterate (GnomeKeyringAsk *ask)
 	}
 }
 
-
-static gboolean
-idle_ask_iterate (gpointer data)
-{
-	gnome_keyring_ask_iterate ((GnomeKeyringAsk *)data);
-	return FALSE;
-}
-
 gpointer
 gnome_keyring_ask (GList                             *access_requests,
 		   GnomeKeyringApplicationRef        *app_ref,
@@ -2670,14 +2815,15 @@ GnomeKeyringOperationImplementation keyring_ops[] = {
 	{ op_find_collect, op_find_execute }, /* FIND */
 	{ op_create_item_collect, op_create_item_execute}, /* CREATE_ITEM */
 	{ op_delete_item_collect, op_delete_item_execute}, /* DELETE_ITEM */
-	{ op_get_item_info_or_attributes_collect, op_get_item_info_execute}, /* GET_ITEM_INFO */
+	{ op_get_item_info_collect, op_get_item_info_execute}, /* GET_ITEM_INFO */
 	{ op_set_item_info_or_attributes_collect, op_set_item_info_execute}, /* SET_ITEM_INFO */
-	{ op_get_item_info_or_attributes_collect, op_get_item_attributes_execute}, /* GET_ITEM_ATTRIBUTES */
+	{ op_get_item_acl_or_attributes_collect, op_get_item_attributes_execute}, /* GET_ITEM_ATTRIBUTES */
 	{ op_set_item_info_or_attributes_collect, op_set_item_attributes_execute}, /* SET_ITEM_ATTRIBUTES */
-	{ op_get_item_info_or_attributes_collect, op_get_item_acl_execute}, /* GET_ITEM_ACL */
+	{ op_get_item_acl_or_attributes_collect, op_get_item_acl_execute}, /* GET_ITEM_ACL */
 	{ op_set_item_info_or_attributes_collect, op_set_item_acl_execute}, /* SET_ITEM_ACL */
 	{ op_change_keyring_password_collect, op_change_keyring_password_execute }, /*CHANGE_KEYRING_PASSWORD*/
  	{ NULL, op_set_daemon_display_execute}, /* SET_DAEMON_DISPLAY */
+	{ op_get_item_info_collect, op_get_item_info_execute}, /* GET_ITEM_INFO_PARTIAL */
 };
 
 static RETSIGTYPE
